package nemosofts.hd.wallpaper.activity;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.Toolbar;
import androidx.nemosofts.AppCompat;
import androidx.nemosofts.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.StaggeredGridLayoutManager;

import com.google.android.material.floatingactionbutton.FloatingActionButton;

import java.util.ArrayList;
import java.util.Collections;

import nemosofts.hd.wallpaper.R;
import nemosofts.hd.wallpaper.adapter.AdapterColors;
import nemosofts.hd.wallpaper.adapter.AdapterWallpaper;
import nemosofts.hd.wallpaper.asyncTask.LoadWallpaper;
import nemosofts.hd.wallpaper.callback.Callback;
import nemosofts.hd.wallpaper.dialog.VerifyDialog;
import nemosofts.hd.wallpaper.interfaces.WallpaperListener;
import nemosofts.hd.wallpaper.item.ItemWallpaper;
import nemosofts.hd.wallpaper.utils.IfSupported;
import nemosofts.hd.wallpaper.utils.SharedPref;
import nemosofts.hd.wallpaper.utils.helper.Helper;
import nemosofts.hd.wallpaper.utils.recycler.EndlessRecyclerViewScrollListener;
import nemosofts.hd.wallpaper.utils.recycler.RecyclerItemClickListener;

public class PostIDActivity extends AppCompatActivity {

    private Helper helper;
    private RecyclerView rv;
    private AdapterWallpaper adapter;
    private ArrayList<ItemWallpaper> arrayList;
    private int page = 1,  nativeAdPos = 0;
    private Boolean isOver = false, isScroll = false, isLoading = false;
    private StaggeredGridLayoutManager grid;
    private ProgressBar pb;
    private FloatingActionButton fab;
    private String error_msg;
    private FrameLayout frameLayout;
    private String color_ids = "";
    private String cid = "", name = "";

    @SuppressLint("NotifyDataSetChanged")
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        IfSupported.IsRTL(this);
        IfSupported.IsScreenshot(this);

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        if (getSupportActionBar() != null) {
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);
            getSupportActionBar().setDisplayShowHomeEnabled(true);
        }
        toolbar.setNavigationOnClickListener(view -> finish());

        Intent intent = getIntent();
        cid = intent.getStringExtra("id");
        name = intent.getStringExtra("name");

        setTitle(name);

        grid = new StaggeredGridLayoutManager(2, StaggeredGridLayoutManager.VERTICAL);
        grid.setGapStrategy(StaggeredGridLayoutManager.GAP_HANDLING_MOVE_ITEMS_BETWEEN_SPANS);

        helper = new Helper(this, (position, type) -> {
            Callback.arrayList.clear();
            Callback.arrayList.addAll(arrayList);
            Callback.arrayList.removeAll(Collections.singleton(null));

            int real_pos = Callback.arrayList.indexOf(arrayList.get(position));

            Intent intent1 = new Intent(this, WallpaperDetailsActivity.class);
            intent1.putExtra("pos", real_pos);
            intent1.putExtra("pos_type", 3);
            intent1.putExtra("page", page);
            intent1.putExtra("color_ids", color_ids);
            startActivity(intent1);
        });

        arrayList = new ArrayList<>();

        frameLayout = findViewById(R.id.fl_empty);
        fab = findViewById(R.id.fab);
        pb = findViewById(R.id.pb);
        rv = findViewById(R.id.rv);
        rv.setHasFixedSize(true);
        rv.setLayoutManager(grid);
        rv.addOnScrollListener(new EndlessRecyclerViewScrollListener(grid) {
            @Override
            public void onLoadMore(int p, int totalItemsCount) {
                if ((Boolean.TRUE.equals(!isOver) && Boolean.TRUE.equals(!isLoading))) {
                    new Handler().postDelayed(() -> {
                        isScroll = true;
                        getData();
                    }, 0);
                }
            }
        });
        rv.addOnScrollListener(new RecyclerView.OnScrollListener() {
            @Override
            public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
                super.onScrolled(recyclerView, dx, dy);
                super.onScrolled(recyclerView, dx, dy);
                int[] firstVisibleItem = grid.findFirstVisibleItemPositions(null);
                if (firstVisibleItem[0] > 6) {
                    fab.show();
                } else {
                    fab.hide();
                }
            }
        });
        fab.setOnClickListener(v -> rv.smoothScrollToPosition(0));

        if(Callback.nativeAdShow%2 != 0) {
            nativeAdPos = Callback.nativeAdShow + 1;
        } else {
            nativeAdPos = Callback.nativeAdShow;
        }
        getData();

        if (Boolean.TRUE.equals(Callback.isColorOn) && !Callback.arrayListColors.isEmpty()) {
            RecyclerView rv_colors = findViewById(R.id.rv_wall_colors);
            rv_colors.setHasFixedSize(true);
            LinearLayoutManager llm = new LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false);
            rv_colors.setLayoutManager(llm);
            rv_colors.setNestedScrollingEnabled(false);

            AdapterColors adapterColors = new AdapterColors(this, Callback.arrayListColors);
            rv_colors.setAdapter(adapterColors);
            rv_colors.addOnItemTouchListener(new RecyclerItemClickListener(this, (view, position) -> adapterColors.setSelected(position)));

            findViewById(R.id.layout_colors).setVisibility(View.VISIBLE);
            findViewById(R.id.button_colors_go).setOnClickListener(v -> {
                color_ids = adapterColors.getSelected();
                page = 1;
                isOver = false;
                isScroll = false;
                isLoading = false;
                arrayList.clear();
                if(adapter != null) {
                    adapter.notifyDataSetChanged();
                }
                getData();
            });
        } else {
            findViewById(R.id.layout_colors).setVisibility(View.GONE);
        }
    }

    private void getData() {
        if (helper.isNetworkAvailable()) {
            LoadWallpaper loadCategory = new LoadWallpaper(new WallpaperListener() {
                @Override
                public void onStart() {
                    isLoading = true;
                    if (arrayList.isEmpty()) {
                        frameLayout.setVisibility(View.GONE);
                        pb.setVisibility(View.VISIBLE);
                    }
                }

                @Override
                public void onEnd(String success, String verifyStatus, String message, ArrayList<ItemWallpaper> arrayListWallpaper) {
                    if (success.equals("1")) {
                        if (!verifyStatus.equals("-1")) {
                            if (arrayListWallpaper.isEmpty()) {
                                isOver = true;
                                error_msg = getString(R.string.err_no_data_found);
                                setEmpty();
                            } else {
                                for (int i = 0; i < arrayListWallpaper.size(); i++) {
                                    arrayList.add(arrayListWallpaper.get(i));
                                    if (helper.canLoadNativeAds(PostIDActivity.this, Callback.PAGE_NATIVE_POST)) {
                                        int abc = arrayList.lastIndexOf(null);
                                        if (((arrayList.size() - (abc + 1)) % nativeAdPos == 0)) {
                                            arrayList.add(null);
                                        }
                                    }
                                }
                                page = page + 1;
                                setAdapter();
                            }
                        } else {
                            new VerifyDialog(PostIDActivity.this, getString(R.string.err_unauthorized_access), message);
                        }
                    } else {
                        error_msg = getString(R.string.err_server_not_connected);
                        setEmpty();
                    }
                    isLoading = false;
                }
            }, helper.getAPIRequest(Callback.METHOD_WALLPAPER_BY_CAT, page, color_ids, cid, "", "", new SharedPref(this).getUserId(), "", "", "", "", "", "", "", null));
            loadCategory.execute();
        } else {
            error_msg = getString(R.string.err_internet_not_connected);
            setEmpty();
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    public void setAdapter() {
        if(Boolean.FALSE.equals(isScroll)) {
            adapter = new AdapterWallpaper(this, arrayList, position -> helper.showInterAd(position, ""));
            rv.setAdapter(adapter);
            rv.scheduleLayoutAnimation();
            setEmpty();
        } else {
            adapter.notifyDataSetChanged();
        }
    }

    private void setEmpty() {
        if (!arrayList.isEmpty()) {
            rv.setVisibility(View.VISIBLE);
            pb.setVisibility(View.INVISIBLE);
            frameLayout.setVisibility(View.GONE);
        } else {
            rv.setVisibility(View.GONE);
            frameLayout.setVisibility(View.VISIBLE);
            pb.setVisibility(View.INVISIBLE);

            frameLayout.removeAllViews();
            LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);

            @SuppressLint("InflateParams") View myView = inflater.inflate(R.layout.row_empty, null);

            TextView textView = myView.findViewById(R.id.tv_empty_msg);
            textView.setText(error_msg);

            myView.findViewById(R.id.ll_empty_try).setOnClickListener(v -> getData());

            frameLayout.addView(myView);
        }
    }

    @Override
    public void onDestroy() {
        if (adapter != null) {
            adapter.destroyNativeAds();
        }
        super.onDestroy();
    }

    @Override
    public int setLayoutResourceId() {
        return R.layout.activity_post_id;
    }

    @Override
    public int setAppCompat() {
        return AppCompat.COMPAT();
    }
}