package nemosofts.hd.wallpaper.adapter;

import android.content.Context;
import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.nemosofts.theme.ThemeEngine;
import androidx.recyclerview.widget.RecyclerView;

import com.makeramen.roundedimageview.RoundedImageView;
import com.squareup.picasso.Picasso;

import java.util.ArrayList;

import nemosofts.hd.wallpaper.R;
import nemosofts.hd.wallpaper.item.ItemColors;
import nemosofts.hd.wallpaper.utils.ApplicationUtil;

public class AdapterColors extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    private final Context ctx;
    private final ArrayList<ItemColors> arrayList;
    private final ArrayList<String> arrayListSelected = new ArrayList<>();

    private static class MyViewHolder extends RecyclerView.ViewHolder {

        private final RoundedImageView imageView;
        private final RoundedImageView iv_color_tick;

        private MyViewHolder(View view) {
            super(view);
            imageView = view.findViewById(R.id.iv_color);
            iv_color_tick = view.findViewById(R.id.iv_color_tick);
        }
    }

    public AdapterColors(Context context, ArrayList<ItemColors> arrayList) {
        this.arrayList = arrayList;
        this.ctx = context;
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View itemView = LayoutInflater.from(parent.getContext()).inflate(R.layout.row_colors_list, parent, false);
        return new MyViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(@NonNull final RecyclerView.ViewHolder holder, int position) {

        if(arrayListSelected.contains(arrayList.get(position).getId())) {

            ((MyViewHolder) holder).imageView.setBorderColor(Boolean.TRUE.equals(Boolean.TRUE.equals(new ThemeEngine(ctx).getIsThemeMode())) ? Color.WHITE : Color.BLACK);
            ((MyViewHolder) holder).iv_color_tick.setVisibility(View.VISIBLE);

        } else {
            ((MyViewHolder) holder).imageView.setBorderColor(Color.TRANSPARENT);
            ((MyViewHolder) holder).iv_color_tick.setVisibility(View.GONE);
        }

        ((MyViewHolder) holder).imageView.setColorFilter(Color.parseColor(arrayList.get(position).getColorHex()));

        Picasso.get()
            .load(arrayList.get(position).getColorHex())
            .placeholder(R.color.md_classic_background_s)
            .into(((MyViewHolder) holder).imageView);
    }

    public void setSelected(int position) {
        if(arrayListSelected.contains(arrayList.get(position).getId())) {
            arrayListSelected.remove(arrayList.get(position).getId());
        } else {
            arrayListSelected.add(arrayList.get(position).getId());
        }
        notifyItemChanged(position);
    }

    public String getSelected() {
        String selectedIDs="";
        if(!arrayListSelected.isEmpty() && arrayListSelected.size() < getItemCount()) {
            StringBuilder selectedIDsBuilder = new StringBuilder(arrayListSelected.get(0));
            for (int i = 1; i < arrayListSelected.size(); i++) {
                selectedIDsBuilder.append(",").append(arrayListSelected.get(i));
            }
            selectedIDs = selectedIDsBuilder.toString();
        }
        return selectedIDs;
    }

    @Override
    public int getItemCount() {
        return arrayList.size();
    }

    @Override
    public long getItemId(int position) {
        return position;
    }
}