package nemosofts.hd.wallpaper.adapter.home;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.nemosofts.theme.ThemeEngine;
import androidx.recyclerview.widget.RecyclerView;

import com.like.LikeButton;
import com.like.OnLikeListener;
import com.makeramen.roundedimageview.RoundedImageView;
import com.squareup.picasso.Picasso;

import java.util.List;

import nemosofts.hd.wallpaper.R;
import nemosofts.hd.wallpaper.asyncTask.LoadStatus;
import nemosofts.hd.wallpaper.callback.Callback;
import nemosofts.hd.wallpaper.interfaces.RecyclerViewClickListener;
import nemosofts.hd.wallpaper.interfaces.SuccessListener;
import nemosofts.hd.wallpaper.item.ItemWallpaper;
import nemosofts.hd.wallpaper.utils.ApplicationUtil;
import nemosofts.hd.wallpaper.utils.SharedPref;
import nemosofts.hd.wallpaper.utils.helper.Helper;

public class AdapterHomeHorizontal extends RecyclerView.Adapter<AdapterHomeHorizontal.MyViewHolder> {

    Context context;
    Helper helper;
    SharedPref sharedPref;
    List<ItemWallpaper> arrayList;
    RecyclerViewClickListener recyclerViewClickListener;

    static class MyViewHolder extends RecyclerView.ViewHolder {

        LikeButton likeButton;
        TextView tv_title;
        RoundedImageView iv_wallpaper;
        ImageView iv_wallpaper_pre;

        MyViewHolder(View view) {
            super(view);

            iv_wallpaper = view.findViewById(R.id.iv_wallpaper);
            likeButton = view.findViewById(R.id.button_wall_fav);
            tv_title = view.findViewById(R.id.tv_wall_cat);
            iv_wallpaper_pre = view.findViewById(R.id.iv_wallpaper_pre);
        }
    }

    public AdapterHomeHorizontal(Context context, List<ItemWallpaper> arrayList, RecyclerViewClickListener recyclerViewClickListener) {
        this.arrayList = arrayList;
        this.context = context;
        helper = new Helper(context);
        sharedPref = new SharedPref(context);
        this.recyclerViewClickListener = recyclerViewClickListener;
    }

    @NonNull
    @Override
    public MyViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View itemView = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_home_horizontal, parent, false);
        return new MyViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(@NonNull MyViewHolder holder, int position) {

        holder.likeButton.setLiked(arrayList.get(position).getIsFav());
        holder.tv_title.setText(arrayList.get(position).getImageName().isEmpty() ? arrayList.get(position).getCName() : arrayList.get(position).getImageName());

        if (Boolean.FALSE.equals(Callback.isPurchases)){
            holder.iv_wallpaper_pre.setVisibility(arrayList.get(position).isPremium()? View.VISIBLE :View.GONE);
        } else {
            holder.iv_wallpaper_pre.setVisibility(View.GONE);
        }

        Picasso.get()
                .load(arrayList.get(position).getImage())
                .resize(ApplicationUtil.getImageThumbWidth("Home"), ApplicationUtil.getImageThumbHeight("Home"))
                .centerCrop()
                .placeholder(Boolean.TRUE.equals(new ThemeEngine(context).getIsThemeMode()) ? R.drawable.ic_placeholder_home_n : R.drawable.ic_placeholder_home_l)
                .into(holder.iv_wallpaper);

        if (sharedPref.isLogged()) {
            holder.likeButton.setOnLikeListener(new OnLikeListener() {
                @Override
                public void liked(LikeButton likeButton) {
                    try {
                        loadFav(holder.getAdapterPosition());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

                @Override
                public void unLiked(LikeButton likeButton) {
                    try {
                        loadFav(holder.getAdapterPosition());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            });
        } else {
            holder.likeButton.setOnClickListener(view -> {
                if (!sharedPref.isLogged()) {
                    helper.clickLogin();
                }
            });
        }
        holder.iv_wallpaper.setOnClickListener(view -> recyclerViewClickListener.onClick(holder.getAdapterPosition()));
    }

    @Override
    public long getItemId(int id) {
        return id;
    }

    @Override
    public int getItemCount() {
        return arrayList.size();
    }

    private void loadFav(int pos) {
        if (sharedPref.isLogged()) {
            if (helper.isNetworkAvailable()) {
                LoadStatus loadFav = new LoadStatus(new SuccessListener() {
                    @Override
                    public void onStart() {

                    }

                    @Override
                    public void onEnd(String success, String favSuccess, String message) {
                        if (success.equals("1")) {
                            arrayList.get(pos).setIsFav(favSuccess.equals("1"));
                            Toast.makeText(context, message, Toast.LENGTH_SHORT).show();
                        } else {
                            Toast.makeText(context, context.getString(R.string.err_server_not_connected), Toast.LENGTH_SHORT).show();
                        }
                    }
                }, helper.getAPIRequest(Callback.METHOD_DO_FAV, 0, arrayList.get(pos).getId(), "", "", "", sharedPref.getUserId(), "", "", "", "", "", "", "wallpaper", null));
                loadFav.execute();
            } else {
                Toast.makeText(context, context.getString(R.string.err_internet_not_connected), Toast.LENGTH_SHORT).show();
            }
        } else {
            helper.clickLogin();
        }
    }
}