package nemosofts.hd.wallpaper.fragment;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.DefaultItemAnimator;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.makeramen.roundedimageview.RoundedImageView;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Random;

import nemosofts.hd.wallpaper.R;
import nemosofts.hd.wallpaper.adapter.home.AdapterHome;
import nemosofts.hd.wallpaper.asyncTask.LoadHome;
import nemosofts.hd.wallpaper.callback.Callback;
import nemosofts.hd.wallpaper.dialog.InvalidUserDialog;
import nemosofts.hd.wallpaper.interfaces.HomeListener;
import nemosofts.hd.wallpaper.item.ItemPost;
import nemosofts.hd.wallpaper.utils.SharedPref;
import nemosofts.hd.wallpaper.utils.helper.Helper;

public class FragmentHome extends Fragment {

    Helper helper;
    SharedPref sharedPref;
    ProgressBar progressBar;
    FrameLayout frameLayout;
    RecyclerView rv_home;
    AdapterHome adapterHome;
    ArrayList<ItemPost> arrayList;
    private String error_msg;
    RelativeLayout rl_mor;

    private RoundedImageView iv_material;

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_home, container, false);

        helper = new Helper(getActivity());
        sharedPref = new SharedPref(getActivity());

        arrayList = new ArrayList<>();

        progressBar = rootView.findViewById(R.id.pb_home);
        frameLayout = rootView.findViewById(R.id.fl_empty);

        rl_mor = rootView.findViewById(R.id.rl_mor);
        rv_home = rootView.findViewById(R.id.rv_home);
        LinearLayoutManager llm = new LinearLayoutManager(getActivity());
        rv_home.setLayoutManager(llm);
        rv_home.setItemAnimator(new DefaultItemAnimator());
        rv_home.setHasFixedSize(true);

        iv_material = rootView.findViewById(R.id.iv_material);
        TextView titleWelcome = rootView.findViewById(R.id.title_welcome);
        titleWelcome.setText(getTimeOfTheDay());

        loadHome();

        setHasOptionsMenu(true);
        return rootView;
    }

    private String getTimeOfTheDay() {
        String message = getString(R.string.title_good_day);
        Calendar c = Calendar.getInstance();
        int timeOfDay = c.get(Calendar.HOUR_OF_DAY);

        String[] images = new String[]{};
        if (timeOfDay >= 0 && timeOfDay < 6) {
            message = getString(R.string.title_good_night);
            images = getResources().getStringArray(R.array.night);
        } else if (timeOfDay >= 6 && timeOfDay < 12) {
            message = getString(R.string.title_good_morning);
            images = getResources().getStringArray(R.array.morning);
        } else if (timeOfDay >= 12 && timeOfDay < 16) {
            message = getString(R.string.title_good_afternoon);
            images = getResources().getStringArray(R.array.after_noon);
        } else if (timeOfDay >= 16 && timeOfDay < 20) {
            message = getString(R.string.title_good_evening);
            images = getResources().getStringArray(R.array.evening);
        } else if (timeOfDay >= 20 && timeOfDay < 24) {
            message = getString(R.string.title_good_night);
            images = getResources().getStringArray(R.array.night);
        }
        String day = images[new Random().nextInt(images.length)];
        loadTimeImage(day);
        return message;
    }

    private void loadTimeImage(String day) {
        Glide.with(requireContext())
                .load(day)
                .placeholder(R.drawable.material_design_default)
                .diskCacheStrategy(DiskCacheStrategy.AUTOMATIC)
                .into(iv_material);
    }

    private void loadHome() {
        if (helper.isNetworkAvailable()) {
            LoadHome loadHome = new LoadHome(new HomeListener() {
                @Override
                public void onStart() {
                    frameLayout.setVisibility(View.GONE);
                    rv_home.setVisibility(View.GONE);
                    rl_mor.setVisibility(View.GONE);
                    progressBar.setVisibility(View.VISIBLE);
                }

                @Override
                public void onEnd(String success, String message, ArrayList<ItemPost> arrayListPost) {
                    if (getActivity() != null) {
                        if (success.equals("1")) {
                            if (!arrayListPost.isEmpty()){

                                arrayList.addAll(arrayListPost);
                                if (Callback.isAdsStatus){
                                    arrayList.add(new ItemPost("100","ads","ads", "ads"));
                                }
                                adapterHome = new AdapterHome(getActivity(), arrayList);

                                rv_home.setAdapter(adapterHome);
                                setEmpty();
                            } else {
                                error_msg = getString(R.string.err_no_data_found);
                                setEmpty();
                            }

                        } else if (success.equals("-2")) {
                            new InvalidUserDialog(getActivity(), message);
                        } else {
                            error_msg = getString(R.string.err_server);
                            setEmpty();
                        }
                        progressBar.setVisibility(View.GONE);
                    }
                }
            }, helper.getAPIRequest(Callback.METHOD_HOME, 0, "", "", "", "", sharedPref.getUserId(), "", "", "", "", "", "", "", null));
            loadHome.execute();
        } else {
            error_msg = getString(R.string.err_internet_not_connected);
            setEmpty();
        }
    }

    private void setEmpty() {
        if (!arrayList.isEmpty()) {
            rv_home.setVisibility(View.VISIBLE);
            progressBar.setVisibility(View.INVISIBLE);
            frameLayout.setVisibility(View.GONE);
            rl_mor.setVisibility(View.VISIBLE);
        } else {
            rv_home.setVisibility(View.GONE);
            frameLayout.setVisibility(View.VISIBLE);
            progressBar.setVisibility(View.INVISIBLE);
            rl_mor.setVisibility(View.GONE);

            frameLayout.removeAllViews();
            LayoutInflater inflater = (LayoutInflater) requireActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);

            @SuppressLint("InflateParams") View myView = inflater.inflate(R.layout.row_empty, null);

            TextView textView = myView.findViewById(R.id.tv_empty_msg);
            textView.setText(error_msg);

            myView.findViewById(R.id.ll_empty_try).setOnClickListener(v -> loadHome());

            frameLayout.addView(myView);
        }
    }
}