package nemosofts.hd.wallpaper.fragment.online;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.StaggeredGridLayoutManager;

import com.google.android.material.floatingactionbutton.FloatingActionButton;

import java.util.ArrayList;
import java.util.Collections;

import nemosofts.hd.wallpaper.R;
import nemosofts.hd.wallpaper.activity.WallpaperDetailsActivity;
import nemosofts.hd.wallpaper.adapter.AdapterWallpaper;
import nemosofts.hd.wallpaper.asyncTask.LoadWallpaper;
import nemosofts.hd.wallpaper.callback.Callback;
import nemosofts.hd.wallpaper.dialog.VerifyDialog;
import nemosofts.hd.wallpaper.interfaces.WallpaperListener;
import nemosofts.hd.wallpaper.item.ItemWallpaper;
import nemosofts.hd.wallpaper.utils.SharedPref;
import nemosofts.hd.wallpaper.utils.helper.Helper;

public class FragmentSection extends Fragment {

    private Helper helper;
    private RecyclerView rv;
    private AdapterWallpaper adapter;
    private ArrayList<ItemWallpaper> arrayList;
    private int nativeAdPos = 0;
    private StaggeredGridLayoutManager grid;
    private ProgressBar pb;
    private FloatingActionButton fab;
    private String error_msg;
    private FrameLayout frameLayout;
    private String homeSecID = "";

    @SuppressLint("NotifyDataSetChanged")
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_latest, container, false);

        grid = new StaggeredGridLayoutManager(2, StaggeredGridLayoutManager.VERTICAL);
        grid.setGapStrategy(StaggeredGridLayoutManager.GAP_HANDLING_MOVE_ITEMS_BETWEEN_SPANS);

        homeSecID = getArguments().getString("id");

        helper = new Helper(getActivity(), (position, type) -> {
            Callback.arrayList.clear();
            Callback.arrayList.addAll(arrayList);
            Callback.arrayList.removeAll(Collections.singleton(null));

            int real_pos = Callback.arrayList.indexOf(arrayList.get(position));

            Intent intent = new Intent(getActivity(), WallpaperDetailsActivity.class);
            intent.putExtra("pos", real_pos);
            intent.putExtra("pos_type", 0);
            intent.putExtra("page", 0);
            intent.putExtra("color_ids", "");
            startActivity(intent);
        });

        arrayList = new ArrayList<>();

        frameLayout = rootView.findViewById(R.id.fl_empty);
        fab = rootView.findViewById(R.id.fab);
        pb = rootView.findViewById(R.id.pb);
        rv = rootView.findViewById(R.id.rv);
        rv.setHasFixedSize(true);
        rv.setLayoutManager(grid);
        rv.addOnScrollListener(new RecyclerView.OnScrollListener() {
            @Override
            public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
                super.onScrolled(recyclerView, dx, dy);
                super.onScrolled(recyclerView, dx, dy);
                int[] firstVisibleItem = grid.findFirstVisibleItemPositions(null);
                if (firstVisibleItem[0] > 6) {
                    fab.show();
                } else {
                    fab.hide();
                }
            }
        });
        fab.setOnClickListener(v -> rv.smoothScrollToPosition(0));

        if(Callback.nativeAdShow%2 != 0) {
            nativeAdPos = Callback.nativeAdShow + 1;
        } else {
            nativeAdPos = Callback.nativeAdShow;
        }
        getData();

        rootView.findViewById(R.id.layout_colors).setVisibility(View.GONE);
        setHasOptionsMenu(true);
        return rootView;
    }

    private void getData() {
        if (helper.isNetworkAvailable()) {
            LoadWallpaper loadCategory = new LoadWallpaper(new WallpaperListener() {
                @Override
                public void onStart() {
                    if (arrayList.isEmpty()) {
                        frameLayout.setVisibility(View.GONE);
                        pb.setVisibility(View.VISIBLE);
                    }
                }

                @Override
                public void onEnd(String success, String verifyStatus, String message, ArrayList<ItemWallpaper> arrayListWallpaper) {
                    if (getActivity() != null) {
                        if (success.equals("1")) {
                            if (!verifyStatus.equals("-1")) {
                                if (arrayListWallpaper.isEmpty()) {
                                    error_msg = getString(R.string.err_no_data_found);
                                    setEmpty();
                                } else {
                                    for (int i = 0; i < arrayListWallpaper.size(); i++) {
                                        arrayList.add(arrayListWallpaper.get(i));
                                        if (helper.canLoadNativeAds(requireContext(), Callback.PAGE_NATIVE_POST)) {
                                            int abc = arrayList.lastIndexOf(null);
                                            if (((arrayList.size() - (abc + 1)) % nativeAdPos == 0)) {
                                                arrayList.add(null);
                                            }
                                        }
                                    }
                                    setAdapter();
                                }
                            } else {
                                new VerifyDialog(getActivity(), getString(R.string.err_unauthorized_access), message);
                            }
                        } else {
                            error_msg = getString(R.string.err_server_not_connected);
                            setEmpty();
                        }
                    }
                }
            }, helper.getAPIRequest(Callback.METHOD_HOME_DETAILS, 0, homeSecID, "", "", "", new SharedPref(getActivity()).getUserId(), "", "", "", "", "", "", "", null));
            loadCategory.execute();
        } else {
            error_msg = getString(R.string.err_internet_not_connected);
            setEmpty();
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    public void setAdapter() {
        adapter = new AdapterWallpaper(getActivity(), arrayList, position -> helper.showInterAd(position, ""));
        rv.setAdapter(adapter);
        rv.scheduleLayoutAnimation();
        setEmpty();
    }

    private void setEmpty() {
        if (!arrayList.isEmpty()) {
            rv.setVisibility(View.VISIBLE);
            pb.setVisibility(View.INVISIBLE);
            frameLayout.setVisibility(View.GONE);
        } else {
            rv.setVisibility(View.GONE);
            frameLayout.setVisibility(View.VISIBLE);
            pb.setVisibility(View.INVISIBLE);

            frameLayout.removeAllViews();
            LayoutInflater inflater = (LayoutInflater) requireActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);

            @SuppressLint("InflateParams") View myView = inflater.inflate(R.layout.row_empty, null);

            TextView textView = myView.findViewById(R.id.tv_empty_msg);
            textView.setText(error_msg);

            myView.findViewById(R.id.ll_empty_try).setOnClickListener(v -> getData());

            frameLayout.addView(myView);
        }
    }

    @Override
    public void onDestroy() {
        if (adapter != null) {
            adapter.destroyNativeAds();
        }
        super.onDestroy();
    }
}