package nemosofts.hd.wallpaper.fragment.online;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.StaggeredGridLayoutManager;

import com.google.android.material.floatingactionbutton.FloatingActionButton;

import java.util.ArrayList;
import java.util.Collections;

import nemosofts.hd.wallpaper.R;
import nemosofts.hd.wallpaper.activity.WallpaperDetailsActivity;
import nemosofts.hd.wallpaper.adapter.AdapterColors;
import nemosofts.hd.wallpaper.adapter.AdapterWallpaper;
import nemosofts.hd.wallpaper.asyncTask.LoadWallpaper;
import nemosofts.hd.wallpaper.callback.Callback;
import nemosofts.hd.wallpaper.dialog.VerifyDialog;
import nemosofts.hd.wallpaper.interfaces.WallpaperListener;
import nemosofts.hd.wallpaper.item.ItemWallpaper;
import nemosofts.hd.wallpaper.utils.SharedPref;
import nemosofts.hd.wallpaper.utils.helper.Helper;
import nemosofts.hd.wallpaper.utils.recycler.EndlessRecyclerViewScrollListener;
import nemosofts.hd.wallpaper.utils.recycler.RecyclerItemClickListener;

public class FragmentTrending extends Fragment {

    private Helper helper;
    private RecyclerView rv;
    private AdapterWallpaper adapter;
    private ArrayList<ItemWallpaper> arrayList;
    private int page = 1,  nativeAdPos = 0;
    private Boolean isOver = false, isScroll = false, isLoading = false;
    private StaggeredGridLayoutManager grid;
    private ProgressBar pb;
    private FloatingActionButton fab;
    private String error_msg;
    private FrameLayout frameLayout;
    private String color_ids = "";

    @SuppressLint("NotifyDataSetChanged")
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_latest, container, false);

        grid = new StaggeredGridLayoutManager(2, StaggeredGridLayoutManager.VERTICAL);
        grid.setGapStrategy(StaggeredGridLayoutManager.GAP_HANDLING_MOVE_ITEMS_BETWEEN_SPANS);

        helper = new Helper(getActivity(), (position, type) -> {
            Callback.arrayList.clear();
            Callback.arrayList.addAll(arrayList);
            Callback.arrayList.removeAll(Collections.singleton(null));

            int real_pos = Callback.arrayList.indexOf(arrayList.get(position));

            Intent intent = new Intent(getActivity(), WallpaperDetailsActivity.class);
            intent.putExtra("pos", real_pos);
            intent.putExtra("pos_type", 0);
            intent.putExtra("page", page);
            intent.putExtra("color_ids", color_ids);
            startActivity(intent);
        });

        arrayList = new ArrayList<>();

        frameLayout = rootView.findViewById(R.id.fl_empty);
        fab = rootView.findViewById(R.id.fab);
        pb = rootView.findViewById(R.id.pb);
        rv = rootView.findViewById(R.id.rv);
        rv.setHasFixedSize(true);
        rv.setLayoutManager(grid);
        rv.addOnItemTouchListener(new RecyclerItemClickListener(getActivity(), (view, position) -> helper.showInterAd(position, "")));
        rv.addOnScrollListener(new EndlessRecyclerViewScrollListener(grid) {
            @Override
            public void onLoadMore(int p, int totalItemsCount) {
                if (getActivity() != null && (Boolean.TRUE.equals(!isOver) && Boolean.TRUE.equals(!isLoading))) {
                    new Handler().postDelayed(() -> {
                        isScroll = true;
                        getData();
                    }, 0);
                }
            }
        });
        rv.addOnScrollListener(new RecyclerView.OnScrollListener() {
            @Override
            public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
                super.onScrolled(recyclerView, dx, dy);
                super.onScrolled(recyclerView, dx, dy);
                int[] firstVisibleItem = grid.findFirstVisibleItemPositions(null);
                if (firstVisibleItem[0] > 6) {
                    fab.show();
                } else {
                    fab.hide();
                }
            }
        });
        fab.setOnClickListener(v -> rv.smoothScrollToPosition(0));

        if(Callback.nativeAdShow%2 != 0) {
            nativeAdPos = Callback.nativeAdShow + 1;
        } else {
            nativeAdPos = Callback.nativeAdShow;
        }
        getData();

        if (Boolean.TRUE.equals(Callback.isColorOn) && !Callback.arrayListColors.isEmpty()) {
            RecyclerView rv_colors =  rootView.findViewById(R.id.rv_wall_colors);
            rv_colors.setHasFixedSize(true);
            LinearLayoutManager llm = new LinearLayoutManager(getActivity(), LinearLayoutManager.HORIZONTAL, false);
            rv_colors.setLayoutManager(llm);
            rv_colors.setNestedScrollingEnabled(false);

            AdapterColors adapterColors = new AdapterColors(requireContext(), Callback.arrayListColors);
            rv_colors.setAdapter(adapterColors);
            rv_colors.addOnItemTouchListener(new RecyclerItemClickListener(getActivity(), (view, position) -> adapterColors.setSelected(position)));

            rootView.findViewById(R.id.layout_colors).setVisibility(View.VISIBLE);
            rootView.findViewById(R.id.button_colors_go).setOnClickListener(v -> {
                color_ids = adapterColors.getSelected();
                page = 1;
                isOver = false;
                isScroll = false;
                isLoading = false;
                arrayList.clear();
                if(adapter != null) {
                    adapter.notifyDataSetChanged();
                }
                getData();
            });
        } else {
            rootView.findViewById(R.id.layout_colors).setVisibility(View.GONE);
        }
        setHasOptionsMenu(true);
        return rootView;
    }

    private void getData() {
        if (helper.isNetworkAvailable()) {
            LoadWallpaper loadCategory = new LoadWallpaper(new WallpaperListener() {
                @Override
                public void onStart() {
                    isLoading = true;
                    if (arrayList.isEmpty()) {
                        frameLayout.setVisibility(View.GONE);
                        pb.setVisibility(View.VISIBLE);
                    }
                }

                @Override
                public void onEnd(String success, String verifyStatus, String message, ArrayList<ItemWallpaper> arrayListWallpaper) {
                    if (getActivity() != null) {
                        if (success.equals("1")) {
                            if (!verifyStatus.equals("-1")) {
                                if (arrayListWallpaper.isEmpty()) {
                                    isOver = true;
                                    error_msg = getString(R.string.err_no_data_found);
                                    setEmpty();
                                } else {
                                    for (int i = 0; i < arrayListWallpaper.size(); i++) {
                                        arrayList.add(arrayListWallpaper.get(i));

                                        if (helper.canLoadNativeAds(requireContext(), Callback.PAGE_NATIVE_POST)) {
                                            int abc = arrayList.lastIndexOf(null);
                                            if (((arrayList.size() - (abc + 1)) % nativeAdPos == 0)) {
                                                arrayList.add(null);
                                            }
                                        }
                                    }
                                    page = page + 1;
                                    setAdapter();
                                }
                            } else {
                                new VerifyDialog(getActivity(), getString(R.string.err_unauthorized_access), message);
                            }
                        } else {
                            error_msg = getString(R.string.err_server_not_connected);
                            setEmpty();
                        }
                        isLoading = false;
                    }
                }
            }, helper.getAPIRequest(Callback.METHOD_MOST_VIEWED, page, color_ids, "", "", "", new SharedPref(getActivity()).getUserId(), "", "", "", "", "", "", "", null));
            loadCategory.execute();
        } else {
            error_msg = getString(R.string.err_internet_not_connected);
            setEmpty();
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    public void setAdapter() {
        if(Boolean.FALSE.equals(isScroll)) {
            adapter = new AdapterWallpaper(getActivity(), arrayList, position -> {

            });
            rv.setAdapter(adapter);
            rv.scheduleLayoutAnimation();
            setEmpty();
        } else {
            adapter.notifyDataSetChanged();
        }
    }

    private void setEmpty() {
        if (!arrayList.isEmpty()) {
            rv.setVisibility(View.VISIBLE);
            pb.setVisibility(View.INVISIBLE);
            frameLayout.setVisibility(View.GONE);
        } else {
            rv.setVisibility(View.GONE);
            frameLayout.setVisibility(View.VISIBLE);
            pb.setVisibility(View.INVISIBLE);

            frameLayout.removeAllViews();
            LayoutInflater inflater = (LayoutInflater) requireActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);

            @SuppressLint("InflateParams") View myView = inflater.inflate(R.layout.row_empty, null);

            TextView textView = myView.findViewById(R.id.tv_empty_msg);
            textView.setText(error_msg);

            myView.findViewById(R.id.ll_empty_try).setOnClickListener(v -> getData());

            frameLayout.addView(myView);
        }
    }

    @Override
    public void onDestroy() {
        if (adapter != null) {
            adapter.destroyNativeAds();
        }
        super.onDestroy();
    }
}