package nemosofts.hd.wallpaper.utils.helper;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.provider.DocumentsContract;
import android.provider.MediaStore;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.FileProvider;

import com.applovin.mediation.MaxAd;
import com.applovin.mediation.MaxAdListener;
import com.applovin.mediation.MaxError;
import com.applovin.mediation.ads.MaxAdView;
import com.applovin.sdk.AppLovinSdk;
import com.google.ads.mediation.admob.AdMobAdapter;
import com.google.ads.mediation.facebook.FacebookMediationAdapter;
import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdSize;
import com.google.android.gms.ads.AdView;
import com.google.android.gms.ads.FullScreenContentCallback;
import com.google.android.gms.ads.MobileAds;
import com.google.android.material.snackbar.Snackbar;
import com.google.firebase.auth.FirebaseAuth;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonObject;
import com.ironsource.mediationsdk.ISBannerSize;
import com.ironsource.mediationsdk.IronSource;
import com.ironsource.mediationsdk.IronSourceBannerLayout;
import com.ironsource.mediationsdk.logger.IronSourceError;
import com.ironsource.mediationsdk.sdk.InterstitialListener;
import com.startapp.sdk.ads.banner.Banner;
import com.startapp.sdk.adsbase.Ad;
import com.startapp.sdk.adsbase.StartAppAd;
import com.startapp.sdk.adsbase.StartAppSDK;
import com.startapp.sdk.adsbase.adlisteners.AdDisplayListener;
import com.unity3d.ads.IUnityAdsInitializationListener;
import com.unity3d.ads.IUnityAdsShowListener;
import com.unity3d.ads.UnityAds;
import com.unity3d.services.banners.BannerView;
import com.unity3d.services.banners.UnityBannerSize;
import com.wortise.ads.WortiseSdk;
import com.wortise.ads.banner.BannerAd;
import com.wortise.ads.interstitial.InterstitialAd;
import com.yandex.mobile.ads.banner.BannerAdView;
import com.yandex.mobile.ads.common.AdError;
import com.yandex.mobile.ads.common.ImpressionData;
import com.yandex.mobile.ads.interstitial.InterstitialAdEventListener;

import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOUtils;
import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;

import javax.net.ssl.HttpsURLConnection;

import nemosofts.hd.wallpaper.R;
import nemosofts.hd.wallpaper.activity.SetAsLiveWallpaperService;
import nemosofts.hd.wallpaper.activity.SetWallpaperActivity;
import nemosofts.hd.wallpaper.activity.SignInActivity;
import nemosofts.hd.wallpaper.callback.Callback;
import nemosofts.hd.wallpaper.interfaces.InterAdListener;
import nemosofts.hd.wallpaper.utils.ApplicationUtil;
import nemosofts.hd.wallpaper.utils.EventAction;
import nemosofts.hd.wallpaper.utils.GlobalBus;
import nemosofts.hd.wallpaper.utils.SharedPref;
import nemosofts.hd.wallpaper.utils.advertising.AdManagerInterAdmob;
import nemosofts.hd.wallpaper.utils.advertising.AdManagerInterApplovin;
import nemosofts.hd.wallpaper.utils.advertising.AdManagerInterStartApp;
import nemosofts.hd.wallpaper.utils.advertising.AdManagerInterUnity;
import nemosofts.hd.wallpaper.utils.advertising.AdManagerInterWortise;
import nemosofts.hd.wallpaper.utils.advertising.AdManagerInterYandex;
import nemosofts.hd.wallpaper.utils.advertising.GDPRChecker;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.RequestBody;

public class Helper {

    private final Context ctx;
    private InterAdListener interAdListener;

    public Helper(Context ctx) {
        this.ctx = ctx;
    }

    public Helper(Context ctx, InterAdListener interAdListener) {
        this.ctx = ctx;
        this.interAdListener = interAdListener;
    }

    public boolean isNetworkAvailable() {
        ConnectivityManager connectivityManager = (ConnectivityManager) ctx.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetworkInfo = connectivityManager.getActiveNetworkInfo();
        return activeNetworkInfo != null && activeNetworkInfo.isConnectedOrConnecting();
    }

    public void initializeAds() {
        if (Callback.adNetwork.equals(Callback.AD_TYPE_ADMOB) || Callback.adNetwork.equals(Callback.AD_TYPE_META)) {
            MobileAds.initialize(ctx, initializationStatus -> {
            });
        }
        if (Callback.adNetwork.equals(Callback.AD_TYPE_STARTAPP)) {
            StartAppSDK.init(ctx, Callback.startappAppID, false);
            StartAppAd.disableSplash();
            StartAppSDK.setUserConsent(ctx, "pas", System.currentTimeMillis(), new GDPRChecker(ctx).canLoadAd());
        }
        if (Callback.adNetwork.equals(Callback.AD_TYPE_APPLOVIN) && (!AppLovinSdk.getInstance(ctx).isInitialized())) {
            AppLovinSdk.initializeSdk(ctx);
            AppLovinSdk.getInstance(ctx).setMediationProvider("max");
            AppLovinSdk.getInstance(ctx).getSettings().setTestDeviceAdvertisingIds(Arrays.asList("656822d9-18de-4120-994e-44d4245a4d63", "249d75a2-1ef2-8ff9-8885-c50384843a66"));
        }
        if (Callback.adNetwork.equals(Callback.AD_TYPE_IRONSOURCE)) {
            IronSource.init(ctx, Callback.ironsourceAppKey, () -> {
            });
        }
        if (Callback.adNetwork.equals(Callback.AD_TYPE_UNITY)) {
            UnityAds.initialize(ctx, Callback.unityGameID, true, new IUnityAdsInitializationListener() {
                @Override
                public void onInitializationComplete() {
                }

                @Override
                public void onInitializationFailed(UnityAds.UnityAdsInitializationError error, String message) {
                }
            });
        }
        if (Callback.adNetwork.equals(Callback.AD_TYPE_YANDEX)) {
            com.yandex.mobile.ads.common.MobileAds.initialize(ctx, () -> {
            });
        }
        if (Callback.adNetwork.equals(Callback.AD_TYPE_WORTISE) && !WortiseSdk.isInitialized()) {
            WortiseSdk.initialize(ctx, Callback.wortiseAppID);
        }
    }

    public Object showBannerAd(LinearLayout linearLayout, String page) {
        if (isBannerAd(page)){
            switch (Callback.adNetwork) {
                case Callback.AD_TYPE_ADMOB:
                case Callback.AD_TYPE_META:
                    Bundle extras = new Bundle();
                    AdView adViewAdmob = new AdView(ctx);
                    AdRequest adRequest;
                    if (Callback.adNetwork.equals(Callback.AD_TYPE_ADMOB)) {
                        adRequest = new AdRequest.Builder()
                                .addNetworkExtrasBundle(AdMobAdapter.class, extras)
                                .build();
                    } else {
                        adRequest = new AdRequest.Builder()
                                .addNetworkExtrasBundle(AdMobAdapter.class, extras)
                                .addNetworkExtrasBundle(FacebookMediationAdapter.class, extras)
                                .build();
                    }
                    adViewAdmob.setAdUnitId(Callback.admobBannerAdID);
                    adViewAdmob.setAdSize(AdSize.BANNER);
                    linearLayout.addView(adViewAdmob);
                    adViewAdmob.loadAd(adRequest);
                case Callback.AD_TYPE_WORTISE:
                    BannerAd mBannerAd = new BannerAd(ctx);
                    mBannerAd.setAdSize(com.wortise.ads.AdSize.HEIGHT_50);
                    mBannerAd.setAdUnitId(Callback.wortiseBannerAdID);
                    linearLayout.addView(mBannerAd);
                    mBannerAd.loadAd();
                    return mBannerAd;
                case Callback.AD_TYPE_STARTAPP:
                    Banner startAppBanner = new Banner(ctx);
                    startAppBanner.setLayoutParams(new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT));
                    linearLayout.addView(startAppBanner);
                    startAppBanner.loadAd();
                    return startAppBanner;
                case Callback.AD_TYPE_UNITY:
                    BannerView bannerView = new BannerView((Activity) ctx, Callback.unityBannerAdID, new UnityBannerSize(320, 50));
                    linearLayout.addView(bannerView);
                    bannerView.load();
                    return null;
                case Callback.AD_TYPE_APPLOVIN:
                    MaxAdView adView = new MaxAdView(Callback.applovinBannerAdID, ctx);
                    int width = ViewGroup.LayoutParams.MATCH_PARENT;
                    int heightPx = ctx.getResources().getDimensionPixelSize(R.dimen.banner_height);
                    adView.setLayoutParams(new FrameLayout.LayoutParams(width, heightPx));
                    linearLayout.addView(adView);
                    adView.loadAd();
                    return adView;
                case Callback.AD_TYPE_IRONSOURCE:
                    IronSourceBannerLayout iBannerAd  = IronSource.createBanner((Activity) ctx, ISBannerSize.BANNER);
                    linearLayout.addView(iBannerAd);
                    IronSource.loadBanner(iBannerAd);
                    return iBannerAd;
                case Callback.AD_TYPE_YANDEX:
                    BannerAdView yBannerAd = new BannerAdView(ctx);
                    int width2 = ViewGroup.LayoutParams.MATCH_PARENT;
                    int heightPx2 = ctx.getResources().getDimensionPixelSize(R.dimen.banner_height);
                    yBannerAd.setLayoutParams(new FrameLayout.LayoutParams(width2, heightPx2));
                    yBannerAd.setAdUnitId(Callback.yandexBannerAdID);
                    com.yandex.mobile.ads.common.AdRequest yadRequest = new com.yandex.mobile.ads.common.AdRequest.Builder().build();
                    linearLayout.addView(yBannerAd);
                    yBannerAd.loadAd(yadRequest);
                    return yBannerAd;
                default:
                    return null;
            }
        } else {
            return null;
        }
    }

    public void showInterAd(final int pos, final String type) {
        if (isInterAd()){
            switch (Callback.adNetwork) {
                case Callback.AD_TYPE_ADMOB, Callback.AD_TYPE_META -> {

                    final AdManagerInterAdmob adManagerInterAdmob = new AdManagerInterAdmob(ctx);
                    if (adManagerInterAdmob.getAd() != null) {
                        adManagerInterAdmob.getAd().setFullScreenContentCallback(new FullScreenContentCallback() {
                            @Override
                            public void onAdDismissedFullScreenContent() {
                                AdManagerInterAdmob.setAd(null);
                                adManagerInterAdmob.createAd();
                                interAdListener.onClick(pos, type);
                                super.onAdDismissedFullScreenContent();
                            }

                            @Override
                            public void onAdFailedToShowFullScreenContent(@NonNull @NotNull com.google.android.gms.ads.AdError adError) {
                                AdManagerInterAdmob.setAd(null);
                                adManagerInterAdmob.createAd();
                                interAdListener.onClick(pos, type);
                                super.onAdFailedToShowFullScreenContent(adError);
                            }
                        });
                        adManagerInterAdmob.getAd().show((Activity) ctx);
                    } else {
                        AdManagerInterAdmob.setAd(null);
                        adManagerInterAdmob.createAd();
                        interAdListener.onClick(pos, type);
                    }
                }
                case Callback.AD_TYPE_STARTAPP -> {
                    final AdManagerInterStartApp adManagerInterStartApp = new AdManagerInterStartApp(ctx);
                    if (adManagerInterStartApp.getAd() != null && adManagerInterStartApp.getAd().isReady()) {
                        adManagerInterStartApp.getAd().showAd(new AdDisplayListener() {
                            @Override
                            public void adHidden(Ad ad) {
                                AdManagerInterStartApp.setAd(null);
                                adManagerInterStartApp.createAd();
                                interAdListener.onClick(pos, type);
                            }

                            @Override
                            public void adDisplayed(Ad ad) {

                            }

                            @Override
                            public void adClicked(Ad ad) {

                            }

                            @Override
                            public void adNotDisplayed(Ad ad) {
                                AdManagerInterStartApp.setAd(null);
                                adManagerInterStartApp.createAd();
                                interAdListener.onClick(pos, type);
                            }
                        });
                    } else {
                        AdManagerInterStartApp.setAd(null);
                        adManagerInterStartApp.createAd();
                        interAdListener.onClick(pos, type);
                    }
                }
                case Callback.AD_TYPE_UNITY -> {
                    final AdManagerInterUnity adManagerInterUnity = new AdManagerInterUnity();
                    if (AdManagerInterUnity.isAdLoaded) {
                        UnityAds.show((Activity) ctx, Callback.unityInterstitialAdID, new IUnityAdsShowListener() {
                            @Override
                            public void onUnityAdsShowFailure(String placementId, UnityAds.UnityAdsShowError error, String message) {
                                AdManagerInterUnity.setAd();
                                adManagerInterUnity.createAd();
                                interAdListener.onClick(pos, type);
                            }

                            @Override
                            public void onUnityAdsShowStart(String placementId) {

                            }

                            @Override
                            public void onUnityAdsShowClick(String placementId) {

                            }

                            @Override
                            public void onUnityAdsShowComplete(String placementId, UnityAds.UnityAdsShowCompletionState state) {
                                AdManagerInterUnity.setAd();
                                adManagerInterUnity.createAd();
                                interAdListener.onClick(pos, type);
                            }
                        });
                    } else {
                        AdManagerInterUnity.setAd();
                        adManagerInterUnity.createAd();
                        interAdListener.onClick(pos, type);
                    }
                }
                case Callback.AD_TYPE_APPLOVIN -> {
                    final AdManagerInterApplovin adManagerInterApplovin = new AdManagerInterApplovin(ctx);
                    if (adManagerInterApplovin.getAd() != null && adManagerInterApplovin.getAd().isReady()) {
                        adManagerInterApplovin.getAd().setListener(new MaxAdListener() {
                            @Override
                            public void onAdLoaded(@NonNull MaxAd ad) {

                            }

                            @Override
                            public void onAdDisplayed(@NonNull MaxAd ad) {

                            }

                            @Override
                            public void onAdHidden(@NonNull MaxAd ad) {
                                AdManagerInterApplovin.setAd(null);
                                adManagerInterApplovin.createAd();
                                interAdListener.onClick(pos, type);
                            }

                            @Override
                            public void onAdClicked(@NonNull MaxAd ad) {

                            }

                            @Override
                            public void onAdLoadFailed(@NonNull String adUnitId, @NonNull MaxError error) {
                                AdManagerInterApplovin.setAd(null);
                                adManagerInterApplovin.createAd();
                                interAdListener.onClick(pos, type);
                            }

                            @Override
                            public void onAdDisplayFailed(@NonNull MaxAd ad, @NonNull MaxError error) {
                                AdManagerInterApplovin.setAd(null);
                                adManagerInterApplovin.createAd();
                                interAdListener.onClick(pos, type);
                            }
                        });
                        adManagerInterApplovin.getAd().showAd();
                    } else {
                        AdManagerInterStartApp.setAd(null);
                        adManagerInterApplovin.createAd();
                        interAdListener.onClick(pos, type);
                    }
                }
                case Callback.AD_TYPE_IRONSOURCE -> {
                    if (IronSource.isInterstitialReady()) {
                        IronSource.setInterstitialListener(new InterstitialListener() {
                            @Override
                            public void onInterstitialAdReady() {

                            }

                            @Override
                            public void onInterstitialAdLoadFailed(IronSourceError error) {
                                interAdListener.onClick(pos, type);
                            }

                            @Override
                            public void onInterstitialAdOpened() {

                            }

                            @Override
                            public void onInterstitialAdClosed() {
                                interAdListener.onClick(pos, type);
                            }

                            @Override
                            public void onInterstitialAdShowFailed(IronSourceError error) {
                                interAdListener.onClick(pos, type);
                            }

                            @Override
                            public void onInterstitialAdClicked() {
                            }

                            @Override
                            public void onInterstitialAdShowSucceeded() {

                            }
                        });
                        IronSource.showInterstitial();
                    } else {
                        interAdListener.onClick(pos, type);
                    }
                    IronSource.init(ctx, Callback.ironsourceAppKey, IronSource.AD_UNIT.INTERSTITIAL);
                    IronSource.loadInterstitial();
                }
                case Callback.AD_TYPE_YANDEX -> {
                    final AdManagerInterYandex adManagerInterYandex = new AdManagerInterYandex(ctx);
                    if (adManagerInterYandex.getAd() != null) {
                        adManagerInterYandex.getAd().setAdEventListener(new InterstitialAdEventListener() {
                            @Override
                            public void onAdShown() {

                            }

                            @Override
                            public void onAdFailedToShow(@NonNull AdError adError) {
                                interAdListener.onClick(pos, type);
                            }

                            @Override
                            public void onAdDismissed() {
                                AdManagerInterYandex.setAd(null);
                                adManagerInterYandex.createAd();
                                interAdListener.onClick(pos, type);
                            }

                            @Override
                            public void onAdClicked() {

                            }

                            @Override
                            public void onAdImpression(@Nullable ImpressionData impressionData) {

                            }
                        });
                        adManagerInterYandex.getAd().show((Activity) ctx);
                    } else {
                        AdManagerInterYandex.setAd(null);
                        adManagerInterYandex.createAd();
                        interAdListener.onClick(pos, type);
                    }
                }
                case Callback.AD_TYPE_WORTISE -> {
                    final AdManagerInterWortise adManagerInterWortise = new AdManagerInterWortise(ctx);
                    if (adManagerInterWortise.getAd() != null && adManagerInterWortise.getAd().isAvailable()) {
                        adManagerInterWortise.getAd().setListener(new InterstitialAd.Listener() {
                            @Override
                            public void onInterstitialShown(@NonNull InterstitialAd interstitialAd) {

                            }

                            @Override
                            public void onInterstitialLoaded(@NonNull InterstitialAd interstitialAd) {

                            }

                            @Override
                            public void onInterstitialImpression(@NonNull InterstitialAd interstitialAd) {

                            }

                            @Override
                            public void onInterstitialFailedToShow(@NonNull InterstitialAd interstitialAd, @NonNull com.wortise.ads.AdError adError) {
                                AdManagerInterWortise.setAd(null);
                                adManagerInterWortise.createAd();
                                interAdListener.onClick(pos, type);
                            }

                            @Override
                            public void onInterstitialFailedToLoad(@NonNull InterstitialAd interstitialAd, @NonNull com.wortise.ads.AdError adError) {
                                AdManagerInterWortise.setAd(null);
                                adManagerInterWortise.createAd();
                                interAdListener.onClick(pos, type);
                            }

                            @Override
                            public void onInterstitialDismissed(@NonNull InterstitialAd interstitialAd) {
                                AdManagerInterWortise.setAd(null);
                                adManagerInterWortise.createAd();
                                interAdListener.onClick(pos, type);
                            }

                            @Override
                            public void onInterstitialClicked(@NonNull InterstitialAd interstitialAd) {

                            }
                        });
                        adManagerInterWortise.getAd().showAd();
                    } else {
                        AdManagerInterWortise.setAd(null);
                        adManagerInterWortise.createAd();
                        interAdListener.onClick(pos, type);
                    }
                }
                default -> interAdListener.onClick(pos, type);
            }
        } else {
            interAdListener.onClick(pos, type);
        }
    }

    private boolean isInterAd() {
        if (isNetworkAvailable() && Boolean.TRUE.equals(Callback.isInterAd) && Boolean.FALSE.equals(Callback.isPurchases) && Boolean.TRUE.equals(Callback.isAdsStatus) && new GDPRChecker(ctx).canLoadAd()) {
            Callback.adCount = Callback.adCount + 1;
            return Callback.adCount % Callback.interstitialAdShow == 0;
        } else {
            return false;
        }
    }

    private boolean isBannerAd(String page) {
        if (isNetworkAvailable() && Boolean.TRUE.equals(Callback.isAdsStatus) && Boolean.FALSE.equals(Callback.isPurchases) && new GDPRChecker(ctx).canLoadAd()) {
            return switch (page) {
                case Callback.PAGE_HOME -> Callback.isBannerAdHome;
                case Callback.PAGE_POST_DETAILS -> Callback.isBannerAdPostDetails;
                case Callback.PAGE_CAT_DETAILS -> Callback.isBannerAdCatDetails;
                case Callback.PAGE_SEARCH -> Callback.isBannerAdSearch;
                default -> true;
            };
        } else {
            return false;
        }
    }

    public boolean canLoadNativeAds(Context context, String page) {
        if (Boolean.TRUE.equals(Callback.isAdsStatus) && Boolean.FALSE.equals(Callback.isPurchases) && new GDPRChecker(context).canLoadAd()) {
            return switch (page) {
                case Callback.PAGE_NATIVE_POST -> Callback.isNativeAdPost;
                case Callback.PAGE_NATIVE_CAT -> Callback.isNativeAdCat;
                default -> true;
            };
        } else {
            return false;
        }
    }

    public RequestBody getAPIRequest(String helper_name, int page, String itemID, String catID, String searchText, String reportMessage, String userID, String name, String email, String mobile, String gender, String password, String authID, String loginType, File file) {
        Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd' 'HH:mm:ss").create();
        JsonObject jsObj = (JsonObject) new Gson().toJsonTree(gson);
        jsObj.addProperty("helper_name", helper_name);
        jsObj.addProperty("application_id", ctx.getPackageName());

        if (Callback.METHOD_APP_DETAILS.equals(helper_name)){
            jsObj.addProperty("user_id", userID);
        } else if (Callback.METHOD_LOGIN.equals(helper_name)){
            jsObj.addProperty("user_email", email);
            jsObj.addProperty("user_password", password);
            jsObj.addProperty("auth_id", authID);
            jsObj.addProperty("type", loginType);
        } else if (Callback.METHOD_REGISTER.equals(helper_name)){
            jsObj.addProperty("user_name", name);
            jsObj.addProperty("user_email", email);
            jsObj.addProperty("user_phone", mobile);
            jsObj.addProperty("user_gender", gender);
            jsObj.addProperty("user_password", password);
            jsObj.addProperty("auth_id", authID);
            jsObj.addProperty("type", loginType);
        } else if (Callback.METHOD_PROFILE.equals(helper_name)) {
            jsObj.addProperty("user_id", userID);
        } else if (Callback.METHOD_ACCOUNT_DELETE.equals(helper_name)) {
            jsObj.addProperty("user_id", userID);
        } else if (Callback.METHOD_EDIT_PROFILE.equals(helper_name)){
            jsObj.addProperty("user_id", userID);
            jsObj.addProperty("user_name", name);
            jsObj.addProperty("user_email", email);
            jsObj.addProperty("user_phone", mobile);
            jsObj.addProperty("user_password", password);
        } else if (Callback.METHOD_USER_IMAGES_UPDATE.equals(helper_name)){
            jsObj.addProperty("user_id", userID);
            jsObj.addProperty("type", loginType);
        } else if (Callback.METHOD_FORGOT_PASSWORD.equals(helper_name)){
            jsObj.addProperty("user_email", email);
        } else if (Callback.METHOD_NOTIFICATION.equals(helper_name)) {
            jsObj.addProperty("page", String.valueOf(page));
            jsObj.addProperty("user_id", userID);
        } else if (Callback.METHOD_REMOVE_NOTIFICATION.equals(helper_name)) {
            jsObj.addProperty("post_id", itemID);
            jsObj.addProperty("user_id", userID);
        } else if (Callback.METHOD_REPORT.equals(helper_name)) {
            jsObj.addProperty("post_id", itemID);
            jsObj.addProperty("user_id", userID);
            jsObj.addProperty("post_type", loginType);
            jsObj.addProperty("report_msg", reportMessage);
        } else if (Callback.METHOD_GET_RATINGS.equals(helper_name)) {
            jsObj.addProperty("post_id", itemID);
            jsObj.addProperty("device_id", userID);
            jsObj.addProperty("post_type", loginType);
        } else if (Callback.METHOD_RATINGS.equals(helper_name)) {
            jsObj.addProperty("post_id", itemID);
            jsObj.addProperty("device_id", userID);
            jsObj.addProperty("rate", authID);
            jsObj.addProperty("message", reportMessage);
            jsObj.addProperty("post_type", loginType);
        }

        else if (Callback.METHOD_HOME.equals(helper_name)) {
            jsObj.addProperty("user_id", userID);
        } else if (Callback.METHOD_HOME_DETAILS.equals(helper_name)) {
            jsObj.addProperty("id", itemID);
            jsObj.addProperty("user_id", userID);
        } else if (Callback.METHOD_LATEST_WALL.equals(helper_name)) {
            jsObj.addProperty("color_id", itemID);
            jsObj.addProperty("user_id", userID);
            jsObj.addProperty("page", String.valueOf(page));
        } else if (Callback.METHOD_MOST_VIEWED.equals(helper_name)) {
            jsObj.addProperty("color_id", itemID);
            jsObj.addProperty("user_id", userID);
            jsObj.addProperty("page", String.valueOf(page));
        } else if (Callback.METHOD_MOST_RATED.equals(helper_name)) {
            jsObj.addProperty("color_id", itemID);
            jsObj.addProperty("user_id", userID);
            jsObj.addProperty("page", String.valueOf(page));
        } else if (Callback.METHOD_WALLPAPER_BY_CAT.equals(helper_name)) {
            jsObj.addProperty("color_id", itemID);
            jsObj.addProperty("cat_id", catID);
            jsObj.addProperty("user_id", userID);
            jsObj.addProperty("page", String.valueOf(page));
        } else if (Callback.METHOD_FAV_WALL.equals(helper_name)) {
            jsObj.addProperty("fav_type", loginType);
            jsObj.addProperty("color_id", itemID);
            jsObj.addProperty("user_id", userID);
            jsObj.addProperty("page", String.valueOf(page));
        } else if (Callback.METHOD_CAT.equals(helper_name)) {
            jsObj.addProperty("page", String.valueOf(page));
            jsObj.addProperty("search_text", searchText);
            jsObj.addProperty("search_type", loginType);
        } else if (Callback.METHOD_WALL_SINGLE.equals(helper_name)) {
            jsObj.addProperty("wallpaper_id", itemID);
            jsObj.addProperty("user_id", userID);
            jsObj.addProperty("post_type", loginType);
        } else if (Callback.METHOD_DO_FAV.equals(helper_name)) {
            jsObj.addProperty("post_id", itemID);
            jsObj.addProperty("user_id", userID);
            jsObj.addProperty("fav_type", loginType);
        }

        // GIF
        else if (Callback.METHOD_LATEST_GIF.equals(helper_name)) {
            jsObj.addProperty("user_id", userID);
            jsObj.addProperty("page", String.valueOf(page));
            jsObj.addProperty("gif_type", loginType);
        } else if (Callback.METHOD_GIF_SINGLE.equals(helper_name)) {
            jsObj.addProperty("wallpaper_id", itemID);
            jsObj.addProperty("user_id", userID);
            jsObj.addProperty("post_type", loginType);
        }

        else if (Callback.TRANSACTION_URL.equals(helper_name)){
            jsObj.addProperty("planId", itemID);
            jsObj.addProperty("planName", catID);
            jsObj.addProperty("planPrice", searchText);
            jsObj.addProperty("planDuration", reportMessage);
            jsObj.addProperty("planCurrencyCode", name);
            jsObj.addProperty("user_id", userID);
        }

        switch (helper_name) {
            case Callback.METHOD_REGISTER, Callback.METHOD_USER_IMAGES_UPDATE -> {
                final MediaType MEDIA_TYPE_PNG = MediaType.parse("image/*");
                MultipartBody.Builder builder = new MultipartBody.Builder().setType(MultipartBody.FORM);
                if (file != null) {
                    builder.addFormDataPart("image_data", file.getName(), RequestBody.create(MEDIA_TYPE_PNG, file));
                }
                return builder.addFormDataPart("data", ApplicationUtil.toBase64(jsObj.toString())).build();
            }
            default -> {
                return new MultipartBody.Builder().setType(MultipartBody.FORM)
                        .addFormDataPart("data", ApplicationUtil.toBase64(jsObj.toString()))
                        .build();
            }
        }
    }

    public void clickLogin() {
        SharedPref sharePref = new SharedPref(ctx);
        if (sharePref.isLogged()) {
            logout((Activity) ctx, sharePref);
            Toast.makeText(ctx, ctx.getString(R.string.logout_success), Toast.LENGTH_SHORT).show();
        } else {
            Intent intent = new Intent(ctx, SignInActivity.class);
            intent.putExtra("from", "app");
            ctx.startActivity(intent);
        }
    }

    public void logout(Activity activity, @NonNull SharedPref sharePref) {
        if (sharePref.getLoginType().equals(Callback.LOGIN_TYPE_GOOGLE)) {
            FirebaseAuth.getInstance().signOut();
        }
        sharePref.setIsAutoLogin(false);
        sharePref.setIsLogged(false);
        sharePref.setLoginDetails("", "", "", "", "", "", "", false, "", Callback.LOGIN_TYPE_NORMAL);
        Intent intent1 = new Intent(ctx, SignInActivity.class);
        intent1.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK);
        intent1.putExtra("from", "");
        ctx.startActivity(intent1);
        activity.finish();
    }

    @SuppressLint("Range")
    public String getPathImage(Uri uri) {
        try {
            String filePath = "";
            String wholeID = DocumentsContract.getDocumentId(uri);

            // Split at colon, use second item in the array
            String id = wholeID.split(":")[1];

            String[] column = {MediaStore.Images.Media.DATA};

            // where id is equal to
            String sel = MediaStore.Images.Media._ID + "=?";

            Cursor cursor = ctx.getContentResolver().query(MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
                    column, sel, new String[]{id}, null);

            int columnIndex = cursor.getColumnIndex(column[0]);

            if (cursor.moveToFirst()) {
                filePath = cursor.getString(columnIndex);
            }

            cursor.close();
            return filePath;
        } catch (Exception e) {
            e.printStackTrace();
            if (uri == null) {
                return null;
            }
            // try to retrieve the image from the media store first
            // this will only work for images selected from gallery
            String[] projection = {MediaStore.Images.Media.DATA};
            Cursor cursor = ctx.getContentResolver().query(uri, projection, null, null, null);
            if (cursor != null) {
                int column_index = cursor
                        .getColumnIndexOrThrow(MediaStore.Images.Media.DATA);
                cursor.moveToFirst();
                String returnn = cursor.getString(column_index);
                cursor.close();

                if (returnn == null) {
                    String path = null, image_id = null;
                    Cursor cursor2 = ctx.getContentResolver().query(uri, null, null, null, null);
                    if (cursor2 != null) {
                        cursor2.moveToFirst();
                        image_id = cursor2.getString(0);
                        image_id = image_id.substring(image_id.lastIndexOf(":") + 1);
                        cursor2.close();
                    }

                    Cursor cursor3 = ctx.getContentResolver().query(android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI, null, MediaStore.Images.Media._ID + " = ? ", new String[]{image_id}, null);
                    if (cursor3 != null) {
                        cursor3.moveToFirst();
                        path = cursor3.getString(cursor3.getColumnIndex(MediaStore.Images.Media.DATA));
                        cursor3.close();
                    }
                    return path;
                }
                return returnn;
            }
            // this is our fallback here
            return uri.getPath();
        }
    }

    public void saveImage(String img_url, String option, View coordinatorLayout, String postType) {
        new LoadShare(option, postType, coordinatorLayout).execute(img_url, FilenameUtils.getName(img_url));
    }

    public class LoadShare extends AsyncTask<String, String, String> {
        private ProgressDialog pDialog;
        View coordinatorLayout;
        String option, filePath, postType;
        File file;

        LoadShare(String option, String postType, View coordinatorLayout) {
            this.option = option;
            this.postType = postType;
            this.coordinatorLayout = coordinatorLayout;
        }

        @Override
        protected void onPreExecute() {
            pDialog = new ProgressDialog(ctx, R.style.dialogTheme);
            if (option.equals(ctx.getString(R.string.download))) {
                if (postType.equals("wallpaper")) {
                    pDialog.setMessage(ctx.getResources().getString(R.string.downloading_wallpaper));
                } else {
                    pDialog.setMessage(ctx.getResources().getString(R.string.downloading_live_wall));
                }
            } else {
                pDialog.setMessage(ctx.getResources().getString(R.string.please_wait));
            }
            pDialog.setIndeterminate(false);
            pDialog.show();
            super.onPreExecute();
        }

        @Override
        protected String doInBackground(String... strings) {
            String name = strings[1];
            try {
                if (!option.equalsIgnoreCase(ctx.getString(R.string.download))) {
                    filePath = ctx.getExternalCacheDir().getAbsoluteFile().getAbsolutePath() + File.separator + name;
                } else {
                    if (isNetworkAvailable()) {
                        GlobalBus.getBus().postSticky(new EventAction(ctx.getString(R.string.download)));
                    }
                    if (postType.equals("wallpaper")) {
                        filePath = Environment.getExternalStorageDirectory() + File.separator + Environment.DIRECTORY_PICTURES + File.separator + ctx.getString(R.string.app_name) + File.separator + ctx.getString(R.string.wallpapers) + File.separator + name;
                    } else {
                        filePath = Environment.getExternalStorageDirectory() + File.separator + Environment.DIRECTORY_PICTURES + File.separator + ctx.getString(R.string.app_name) + File.separator + ctx.getString(R.string.live_wallpapers) + File.separator + name;
                    }
                }
                file = new File(filePath);
                if (!file.exists()) {
                    URL url = new URL(strings[0]);

                    InputStream inputStream;

                    if (strings[0].contains("https://")) {
                        HttpsURLConnection urlConnection = (HttpsURLConnection) url.openConnection();
                        urlConnection.setRequestProperty("Accept", "*/*");
                        urlConnection.setRequestMethod("GET");
                        urlConnection.connect();
                        inputStream = urlConnection.getInputStream();
                    } else {
                        HttpURLConnection urlConnection = (HttpURLConnection) url.openConnection();
                        urlConnection.setRequestProperty("Accept", "*/*");
                        urlConnection.setRequestMethod("GET");
                        urlConnection.connect();
                        inputStream = urlConnection.getInputStream();
                    }

                    if (option.equalsIgnoreCase(ctx.getString(R.string.download))) {
                        boolean isSaved = false;
                        Bitmap bitmap = BitmapFactory.decodeStream(inputStream);
                        if (postType.equals("wallpaper")) {
                            isSaved = saveImage(bitmap, name, option, postType, null);
                        } else {
                            isSaved = saveImage(bitmap, name, option, postType, IOUtils.toByteArray(inputStream));
                        }

                        if (isSaved) {
                            return "1";
                        } else {
                            return "2";
                        }
                    } else {
                        if (file.createNewFile()) {
                            file.createNewFile();
                        }

                        FileOutputStream fileOutput = new FileOutputStream(file);

                        byte[] buffer = new byte[1024];
                        int bufferLength = 0;
                        while ((bufferLength = inputStream.read(buffer)) > 0) {
                            fileOutput.write(buffer, 0, bufferLength);
                        }
                        fileOutput.close();
                        return "1";
                    }
                } else {
                    return "2";
                }
            } catch (MalformedURLException e) {
                return "0";
            } catch (IOException e) {
                return "0";
            }
        }

        @Override
        protected void onPostExecute(String s) {
            pDialog.dismiss();
            if (option.equals(ctx.getString(R.string.download))) {
                if (s.equals("2")) {
                    if (postType.equals("wallpaper")) {
                        showSnackBar(coordinatorLayout, ctx.getResources().getString(R.string.wallpaper_already_saved));
                    } else {
                        showSnackBar(coordinatorLayout, ctx.getResources().getString(R.string.live_wall_already_saved));
                    }
                } else {
                    if (postType.equals("wallpaper")) {
                        showSnackBar(coordinatorLayout, ctx.getResources().getString(R.string.wallpaper_saved));
                    } else {
                        showSnackBar(coordinatorLayout, ctx.getResources().getString(R.string.live_wall_saved));
                    }
                }
            } else if (option.equals(ctx.getString(R.string.set_wallpaper))) {
                Callback.uri_set = FileProvider.getUriForFile(ctx, ctx.getPackageName().concat(".fileprovider"), file);
                if (postType.equals("wallpaper")) {
                    Intent intent = new Intent(ctx, SetWallpaperActivity.class);
                    ctx.startActivity(intent);
                } else {
                    SetAsLiveWallpaperService.setAsWallPaper(ctx);
                }
            } else if (option.equals(ctx.getString(R.string.share))) {
                Uri contentUri = FileProvider.getUriForFile(ctx, ctx.getPackageName().concat(".fileprovider"), file);
                Intent share = new Intent(Intent.ACTION_SEND);
                share.setDataAndType(contentUri, ctx.getContentResolver().getType(contentUri));
                if (postType.equals("wallpaper")) {
                    share.putExtra(Intent.EXTRA_TEXT, ctx.getString(R.string.get_more_wall) + "\n" + ctx.getString(R.string.app_name) + " - " + "https://play.google.com/store/apps/details?id=" + ctx.getPackageName());
                } else {
                    share.putExtra(Intent.EXTRA_TEXT, ctx.getString(R.string.get_more_live_wall) + "\n" + ctx.getString(R.string.app_name) + " - " + "https://play.google.com/store/apps/details?id=" + ctx.getPackageName());
                }
                share.putExtra(Intent.EXTRA_STREAM, contentUri);
                ctx.startActivity(Intent.createChooser(share, ctx.getString(R.string.share)));
            }
            super.onPostExecute(s);
        }
    }

    private boolean saveImage(Bitmap bitmap, String fileName, String type, String postType, byte[] bytes) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q && type.equalsIgnoreCase(ctx.getString(R.string.download))) {
            ContentValues values = new ContentValues();
            if (postType.equals("wallpaper")) {
                values.put(MediaStore.Images.Media.MIME_TYPE, "image/jpeg");
                values.put(MediaStore.Images.Media.RELATIVE_PATH, Environment.DIRECTORY_PICTURES + "/" + ctx.getString(R.string.app_name) + "/" + ctx.getString(R.string.wallpapers));
            } else {
                values.put(MediaStore.Images.Media.MIME_TYPE, "image/gif");
                values.put(MediaStore.Images.Media.RELATIVE_PATH, Environment.DIRECTORY_PICTURES + "/" + ctx.getString(R.string.app_name) + "/" + ctx.getString(R.string.live_wallpapers));
            }
            values.put(MediaStore.Images.Media.DATE_ADDED, System.currentTimeMillis() / 1000);

            values.put(MediaStore.Images.Media.TITLE, fileName);
            values.put(MediaStore.Images.Media.DISPLAY_NAME, fileName);

            values.put(MediaStore.Images.Media.DATE_TAKEN, System.currentTimeMillis());

            values.put(MediaStore.Images.Media.IS_PENDING, true);

            Uri uri = ctx.getContentResolver().insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, values);
            if (uri != null) {
                try {
                    OutputStream outputStream = ctx.getContentResolver().openOutputStream(uri);
                    bitmap.compress(Bitmap.CompressFormat.JPEG, 100, outputStream);
                    outputStream.close();

                    values.put(MediaStore.Images.Media.IS_PENDING, false);
                    ctx.getContentResolver().update(uri, values, null, null);
                } catch (Exception e) {
                    return false;
                }
                return true;
            } else {
                return false;
            }
        } else {
            File directory;

            if (!type.equals(ctx.getString(R.string.download))) {
                directory = new File(ctx.getExternalCacheDir().getAbsoluteFile().getAbsolutePath() + File.separator);
            } else {
                if (postType.equals("wallpaper")) {
                    directory = new File(Environment.getExternalStorageDirectory() + File.separator + Environment.DIRECTORY_PICTURES + File.separator + ctx.getString(R.string.app_name) + File.separator + ctx.getString(R.string.wallpapers) + File.separator);
                } else {
                    directory = new File(Environment.getExternalStorageDirectory() + File.separator + Environment.DIRECTORY_PICTURES + File.separator + ctx.getString(R.string.app_name) + File.separator + ctx.getString(R.string.live_wallpapers) + File.separator);
                }
            }

            if (!directory.exists()) {
                directory.mkdirs();
            }
            File file = new File(directory, fileName);
            try {
                OutputStream outputStream = new FileOutputStream(file);
                if (postType.equals("wallpaper")) {
                    bitmap.compress(Bitmap.CompressFormat.JPEG, 100, outputStream);
                } else {
                    outputStream.write(bytes);
                    outputStream.flush();
                }
                outputStream.close();

                ContentValues values = new ContentValues();
                values.put(MediaStore.Images.Media.DATA, file.getAbsolutePath());
                return true;
            } catch (Exception e) {
                return false;
            }
        }
    }

    public void showSnackBar(View linearLayout, String message) {
        Snackbar snackbar = Snackbar.make(linearLayout, message, Snackbar.LENGTH_SHORT);
        snackbar.getView().setBackgroundResource(R.drawable.bg_snack_bar_success);
        snackbar.show();
    }
}
